# CSS Scrollbars Module Level 1

## Editor’s Draft, 1 January 1970

Specification Metadata

This version:
  https://drafts.csswg.org/css-scrollbars/
Editors:
  Tantek Çelik (Mozilla)
  Rossen Atanassov (Microsoft)
Issue Tracking:
  GitHub Issues
Suggest an Edit for this Spec:
  GitHub Editor

Copyright © 1970 W3C® (MIT, ERCIM, Keio, Beihang). W3C liability, trademark and permissive document license rules apply.

## Abstract

CSS Scrollbars standardizes the ability to color scrollbars introduced in 2000 by Windows IE 5.5. This is useful when building web applications which use color schemes very different from the appearance of default platform scrollbars.

CSS is a language for describing the rendering of structured documents (such as HTML and XML) on screen, on paper, etc.

## Status of this document

This is a public copy of the editors’ draft. It is provided for discussion only and may change at any moment. Its publication here does not imply endorsement of its contents by W3C. Don’t cite this document other than as work in progress.

GitHub Issues are preferred for discussion of this specification. When filing an issue, please put the text “css-scrollbars” in the title, preferably like this: “[css-scrollbars] …summary of comment…”. All issues and comments are archived, and there is also a historical archive.

This document was produced by the CSS Working Group.

This document was produced by a group operating under the W3C Patent Policy. W3C maintains a public list of any patent disclosures made in connection with the deliverables of the group; that page also includes instructions for disclosing a patent. An individual who has actual knowledge of a patent which the individual believes contains Essential Claim(s) must disclose the information in accordance with section 6 of the W3C Patent Policy.

This document is governed by the 1 March 2019 W3C Process Document.

## 1. Introduction

Note: this specification is maintained in the W3C csswg-drafts GitHub repository as css-scrollbars-1 .

### 1.1. Scope

This subsection is non-normative.

Based on documented use-cases, there are three main use-cases around scrollbars this module intends to resolve:

1. Color scrollbars to fit better into the UI of a web application.
2. Use thinner scrollbar when the scrolling area is small.
3. Build customized scrollbars without affecting scrollability.

For addressing these use-cases, this module adds properties to control colors and width of scrollbars (if any) of an element.

Pseudo-elements for selecting specific parts of a scrollbar is out of scope. The WebKit implementation of pseudo-elements for scrollbar is considered to be a feature mistakenly exposed to the web. Main concerns against the pseudo-elements approach are:

- Operating systems continuously evolve their scrollbar designs to provide better user experience, beyond the ability of any set of pseudo-elements to accurately model this over time.
- Different platforms have different scrollbar structure means testing interop is harder, because authors would need to take not only engine but also platforms into account.

### 1.2. Values

This specification follows the CSS property definition conventions from [CSS2]. Value types not defined in this specification are defined in CSS Values & Units [CSS-VALUES-3]. Other CSS modules may expand the definitions of these value types.

In addition to the property-specific values listed in their definitions, all properties defined in this specification also accept the CSS-wide keywords keywords as their property value. For readability they have not been repeated explicitly.

## 2. Scrollbar Colors: the scrollbar-color property

Name: | scrollbar-color
Value: | auto | dark | light | <color>{2}
Initial: | auto
Applies to: | boxes to which overflow applies
Inherited: | yes
Percentages: | n/a
Computed value: | specified keyword or two computed colors
Canonical order: | per grammar
Animation type: | by computed value

This property allows the author to set colors of an element’s scrollbars.

UAs must apply the scrollbar-color value set on the root element to the viewport.

Note: Unlike overflow (and overflow-*) properties, scrollbar-color value set on the HTML body element are not propagated to the viewport.

auto
  default platform rendering for the scrollbar.
dark
  show a dark scrollbar, which can be either a dark variant of scrollbar provided by the platform, or a custom scrollbar with dark colors.
light
  show a light scrollbar, which can be either a light variant of scrollbar provided by the platform, or a custom scrollbar with light colors.
<color>
  apply the first color to the thumb of the scrollbar, and the second color to the track of the scrollbar.

Details:

Track refers to the background of the scrollbar, which is generally fixed regardless of the scrolling position.

Thumb refers to the moving part of the scrollbar, which usually floats on top of the track.

If this property computes to value other than auto, implementations may render a simpler scrollbar than the default platform UI rendering, and color it accordingly.

(Note: add diagram showing the different named pieces - something like http://www.howtocreate.co.uk/tutorials/scrlbar.html)

(Note: add example of an overflow element with colorized scrollbars to match page styling, PNG of the same in a browser that supports it currently)

Implementations may ignore any of the colors if the corresponding part do not exist on the underlying platform.

When using scrollbar-color property with specific color values, authors should ensure the specified colors have enough contrast between them. For keyword values, UAs should ensure the colors they use have enough contrast. See Techniques for WCAG 2.0: G183: Using a contrast ratio of 3:1 […] [WCAG20].

Note: when a user interacts with a scrollbar (e.g. hovering or activating), implementations may alter which scrollbar colors apply to which scrollbar parts.

Note: IE uses named System Colors as defaults for each of the scrollbar color properties. See related Issue 1956.

The following example (derived from https://www.w3.org/Style/Examples/007/scrollbars.en.html) resets scrollbar colors in IE.

    html {
        scrollbar-color: ThreeDFace Scrollbar;
    }

## 3. Scrollbar Thickness: the scrollbar-width property

Name: | scrollbar-width
Value: | auto | thin | none
Initial: | auto
Applies to: | boxes to which overflow applies
Inherited: | no
Percentages: | n/a
Computed value: | specified keyword or absolute length
Canonical order: | per grammar
Animation type: | by computed value

This property allows the author to set the maximum thickness of an element’s scrollbars when they are shown.

auto
  implementations must use the default platform scrollbar width.
thin
  Implementations should use thinner scrollbars than auto when applicable. This may mean a thin variant of scrollbar provided by the platform, or a custom scrollbar thinner than the default platform scrollbar.

  Note: Some platforms may only have a tiny scrollbar by default which cannot be or makes no sense to make thinner. In that case, implementations may treat this value as auto.

none
  implementations must not display any scrollbar, however the element’s scrollability is not affected.

UAs must apply the scrollbar-width value set on the root element to the viewport.

Note: Unlike overflow (and overflow-*) properties, a scrollbar-width value set on the HTML body element is not propagated to the viewport.

Note: This specification does not define the exact position or shape of the scrollbar, or any animation thereof, such as fading or sliding in/out of view.

## Appendix A. Acknowledgments

This appendix is informative.

Thanks to the use-cases, prototyping, implementation, and feedback from Tab Atkins and Xidorn Quan.

## Appendix B. Changes

This appendix is informative.

This is the First Public Working Draft, no previous draft to describe changes from.

## Appendix C. Considerations for Security and Privacy

This appendix is informative.

Per the Self-Review Questionnaire: Security and Privacy: Questions to Consider

1. Does this specification deal with personally-identifiable information?

   No.

2. Does this specification deal with high-value data?

   No.

3. Does this specification introduce new state for an origin that persists across browsing sessions?

   No.

4. Does this specification expose persistent, cross-origin state to the web?

   No.

5. Does this specification expose any other data to an origin that it doesn’t currently have access to?

   No.

6. Does this specification enable new script execution/loading mechanisms?

   No.

7. Does this specification allow an origin access to a user’s location?

   No.

8. Does this specification allow an origin access to sensors on a user’s device?

   No.

9. Does this specification allow an origin access to aspects of a user’s local computing environment?

   No.

10. Does this specification allow an origin access to other devices?

    No.

11. Does this specification allow an origin some measure of control over a user agent’s native UI?

    Yes. The scrollbar-* properties enable the page to change the color and width of the scrollbar of the user agent’s native UI, e.g. scrollbars on the page’s window, on framed content embedded in the page, or on overflowing elements with scrollbars in the page.

12. Does this specification expose temporary identifiers to the web?

    No.

13. Does this specification distinguish between behavior in first-party and third-party contexts?

    No.

14. How should this specification work in the context of a user agent’s "incognito" mode?

    No differently.

15. Does this specification persist data to a user’s local device?

    No.

16. Does this specification have a "Security Considerations" and "Privacy Considerations" section?

    Yes.

17. Does this specification allow downgrading default security characteristics?

    No.

## Conformance

### Document conventions

Conformance requirements are expressed with a combination of descriptive assertions and RFC 2119 terminology. The key words “MUST”, “MUST NOT”, “REQUIRED”, “SHALL”, “SHALL NOT”, “SHOULD”, “SHOULD NOT”, “RECOMMENDED”, “MAY”, and “OPTIONAL” in the normative parts of this document are to be interpreted as described in RFC 2119. However, for readability, these words do not appear in all uppercase letters in this specification.

All of the text of this specification is normative except sections explicitly marked as non-normative, examples, and notes. [RFC2119]

Examples in this specification are introduced with the words “for example” or are set apart from the normative text with `class="example"`, like this:

This is an example of an informative example.

Informative notes begin with the word “Note” and are set apart from the normative text with `class="note"`, like this:

Note, this is an informative note.

Advisements are normative sections styled to evoke special attention and are set apart from other normative text with `<strong class="advisement">`, like this: UAs MUST provide an accessible alternative.

### Conformance classes

Conformance to this specification is defined for three conformance classes:

style sheet
  A CSS style sheet.
renderer
  A UA that interprets the semantics of a style sheet and renders documents that use them.
authoring tool
  A UA that writes a style sheet.

A style sheet is conformant to this specification if all of its statements that use syntax defined in this module are valid according to the generic CSS grammar and the individual grammars of each feature defined in this module.

A renderer is conformant to this specification if, in addition to interpreting the style sheet as defined by the appropriate specifications, it supports all the features defined by this specification by parsing them correctly and rendering the document accordingly. However, the inability of a UA to correctly render a document due to limitations of the device does not make the UA non-conformant. (For example, a UA is not required to render color on a monochrome monitor.)

An authoring tool is conformant to this specification if it writes style sheets that are syntactically correct according to the generic CSS grammar and the individual grammars of each feature in this module, and meet all other conformance requirements of style sheets as described in this module.

### Requirements for Responsible Implementation of CSS

The following sections define several conformance requirements for implementing CSS responsibly, in a way that promotes interoperability in the present and future.

#### Partial Implementations

So that authors can exploit the forward-compatible parsing rules to assign fallback values, CSS renderers must treat as invalid (and ignore as appropriate) any at-rules, properties, property values, keywords, and other syntactic constructs for which they have no usable level of support. In particular, user agents must not selectively ignore unsupported property values and honor supported values in a single multi-value property declaration: if any value is considered invalid (as unsupported values must be), CSS requires that the entire declaration be ignored.

#### Implementations of Unstable and Proprietary Features

To avoid clashes with future stable CSS features, the CSSWG recommends following best practices for the implementation of unstable features and proprietary extensions to CSS.

#### Implementations of CR-level Features

Once a specification reaches the Candidate Recommendation stage, implementers should release an unprefixed implementation of any CR-level feature they can demonstrate to be correctly implemented according to spec, and should avoid exposing a prefixed variant of that feature.

To establish and maintain the interoperability of CSS across implementations, the CSS Working Group requests that non-experimental CSS renderers submit an implementation report (and, if necessary, the testcases used for that implementation report) to the W3C before releasing an unprefixed implementation of any CSS features. Testcases submitted to W3C are subject to review and correction by the CSS Working Group.

Further information on submitting testcases and implementation reports can be found from on the CSS Working Group’s website at http://www.w3.org/Style/CSS/Test/. Questions should be directed to the public-css-testsuite@w3.org mailing list.

## Index

### Terms defined by this specification

- auto
  - value for scrollbar-color, in §2
  - value for scrollbar-width, in §3
- <color>, in §2
- dark, in §2
- light, in §2
- none, in §3
- scrollbar-color, in §2
- scrollbar-width, in §3
- thin, in §3

https://drafts.csswg.org/css-color-3/#valuea-def-color Referenced in:

- 2. Scrollbar Colors: the scrollbar-color property (2)

https://drafts.csswg.org/css-overflow-3/#propdef-overflow Referenced in:

- 2. Scrollbar Colors: the scrollbar-color property (2)
- 3. Scrollbar Thickness: the scrollbar-width property (2)

https://drafts.csswg.org/css-values-4/#css-wide-keywords Referenced in:

- 1.2. Values

https://drafts.csswg.org/css-values-4/#mult-num Referenced in:

- 2. Scrollbar Colors: the scrollbar-color property

https://drafts.csswg.org/css-values-4/#comb-one Referenced in:

- 2. Scrollbar Colors: the scrollbar-color property (2) (3)
- 3. Scrollbar Thickness: the scrollbar-width property (2)

### Terms defined by reference

- [css-color-3] defines the following terms:
  - <color>
- [css-overflow-3] defines the following terms:
  - overflow
- [css-values-4] defines the following terms:
  - css-wide keywords
  - {a}
  - |

## References

### Normative References

[CSS-COLOR-3]
  Tantek Çelik; Chris Lilley; David Baron. CSS Color Module Level 3. 19 June 2018. REC. URL: https://www.w3.org/TR/css-color-3/
[CSS-OVERFLOW-3]
  David Baron; Elika Etemad; Florian Rivoal. CSS Overflow Module Level 3. 31 July 2018. WD. URL: https://www.w3.org/TR/css-overflow-3/
[CSS-VALUES-3]
  Tab Atkins Jr.; Elika Etemad. CSS Values and Units Module Level 3. 31 January 2019. CR. URL: https://www.w3.org/TR/css-values-3/
[CSS-VALUES-4]
  Tab Atkins Jr.; Elika Etemad. CSS Values and Units Module Level 4. 31 January 2019. WD. URL: https://www.w3.org/TR/css-values-4/
[CSS2]
  Bert Bos; et al. Cascading Style Sheets Level 2 Revision 1 (CSS 2.1) Specification. 7 June 2011. REC. URL: https://www.w3.org/TR/CSS2/
[RFC2119]
  S. Bradner. Key words for use in RFCs to Indicate Requirement Levels. March 1997. Best Current Practice. URL: https://tools.ietf.org/html/rfc2119

### Informative References

[WCAG20]
  Ben Caldwell; et al. Web Content Accessibility Guidelines (WCAG) 2.0. 11 December 2008. REC. URL: https://www.w3.org/TR/WCAG20/

## Property Index

Name | Value | Initial | Applies to | Inh. | %ages | Animation type | Canonical order | Computed value
--- | --- | --- | --- | --- | --- | --- | --- | ---
scrollbar-color | auto | dark | light | <color>{2} | auto | boxes to which overflow applies | yes | n/a | by computed value | per grammar | specified keyword or two computed colors
scrollbar-width | auto | thin | none | auto | boxes to which overflow applies | no | n/a | by computed value | per grammar | specified keyword or absolute length

#propdef-scrollbar-color Referenced in:

- 2. Scrollbar Colors: the scrollbar-color property

#valdef-scrollbar-color-auto Referenced in:

- 2. Scrollbar Colors: the scrollbar-color property

#propdef-scrollbar-width Referenced in:

- 3. Scrollbar Thickness: the scrollbar-width property

#valdef-scrollbar-width-auto Referenced in:

- 3. Scrollbar Thickness: the scrollbar-width property (2)